<?php

/*************************************************************************
 * Username Change - XenConcept (c) 2017
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\UsernameChange;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

    public function installStep1()
    {
        $this->schemaManager()->createTable('xc_change_username_logs', function (Create $table)
        {
            $table->addColumn('log_id', 'int')->autoIncrement();
            $table->addColumn('user_id', 'int');
            $table->addColumn('old_username', 'varchar', 100);
            $table->addColumn('new_username', 'varchar', 100);
            $table->addColumn('log_date', 'int');
            $table->addColumn('is_private', 'int')->setDefault(0);
            $table->addColumn('change_state', 'enum')->values(['visible','moderated'])->setDefault('visible');
        });
    }

    public function installStep2()
    {
        $this->schemaManager()->alterTable('xf_user', function (Alter $table)
        {
            $table->addColumn('last_username_change', 'int')->setDefault(0);
        });
    }

    public function installStep3()
    {
        $this->schemaManager()->alterTable('xf_user_privacy', function (Alter $table) {
            $table->addColumn('allow_view_history_username_change', 'enum')->values(['everyone','members','followed','none'])->setDefault('everyone');
        });
    }

    public function installStep4()
    {
        $this->schemaManager()->alterTable('xf_user_option', function (Alter $table)
        {
            $table->addColumn('receive_change_username_email', 'tinyint', 3)->setDefault(1);
        });
    }

    public function upgrade2000600Step1()
    {
        $sm = $this->schemaManager();

        // delete column conflict

        if ($sm->columnExists('xc_change_username_logs','change_state__conflict'))
        {
            $sm->alterTable('xc_change_username_logs', function (Alter $table)
            {
                $table->dropColumns('change_state__conflict');
            });
        }
    }

    public function upgrade2001100Step1()
    {
        $this->schemaManager()->alterTable('xf_user', function (Alter $table)
        {
            $table->addColumn('last_username_change', 'int')->setDefault(0);
        });

        $this->schemaManager()->alterTable('xf_user_privacy', function (Alter $table) {
            $table->addColumn('allow_view_history_username_change', 'enum')->values(['everyone','members','followed','none'])->setDefault('everyone');
        });
    }

    public function upgrade2001700Step1()
    {
        $this->schemaManager()->alterTable('xf_user_option', function (Alter $table)
        {
            $table->addColumn('receive_change_username_email', 'tinyint', 3)->setDefault(1);
        });
    }

    public function uninstallStep1()
    {
        $this->schemaManager()->dropTable('xc_change_username_logs');
    }

    public function uninstallStep2()
    {
        $this->schemaManager()->alterTable('xf_user', function (Alter $table)
        {
            $table->dropColumns('last_username_change');
        });
    }

    public function uninstallStep3()
    {
        $this->schemaManager()->alterTable('xf_user_privacy', function (Alter $table) {
            $table->dropColumns('allow_view_history_username_change');
        });
    }

    public function uninstallStep4()
    {
        $this->schemaManager()->alterTable('xf_user_option', function (Alter $table)
        {
            $table->dropColumns('receive_change_username_email');
        });
    }
}