<?php

/*************************************************************************
 * Username Change - XenConcept (c) 2017
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\UsernameChange\XF\Pub\Controller;

use XF\Mvc\FormAction;

class Account extends XFCP_Account
{

    /**
     * @param  \XenConcept\UsernameChange\XF\Entity\User $user
     */
    protected function setupUsernameChange($user)
    {
        /** @var \XenConcept\UsernameChange\Service\User\UsernameChange $usernameChange */
        $usernameChange = $this->service('XenConcept\UsernameChange:User\UsernameChange', $user);

        if ($user->canUseUsernameChangePrivate())
        {
            $usernameChange->getChangeLog()->is_private = $this->filter('is_private', 'uint');
        }

        $usernameChange->getChangeLog()->new_username = $this->filter('new_username', 'str');

        $usernameChange->getChangeLog()->change_state = $user->canBypassApprovalChange() ? 'visible' : 'moderated';

        return $usernameChange;
    }

    public function actionUsernameChange()
    {
        /** @var \XenConcept\UsernameChange\XF\Entity\User $visitor */
        $visitor = \XF::visitor();

        if (!$visitor->canUsernameChange())
        {
            return $this->noPermission();
        }

        if ($this->isPost())
        {
            $usernameChange = $this->setupUsernameChange($visitor);

            if (!$usernameChange->validate($errors))
            {
                return $this->error($errors);
            }

            $usernameChange->save();

            return $this->redirect($this->buildLink('account/username-change'));
        }
        else
        {
            $disabledButton = false;
            $errorRender    = '';

            if (!$visitor->canUsernameChange(true, $error))
            {
                $disabledButton = true;
                $errorRender    = $error;
            }

            if (!$disabledButton && $visitor->has_username_change_pending)
            {
                $disabledButton = true;
                $errorRender    = \XF::phraseDeferred('xc_username_change_have_a_username_change_pending_approval');
            }

            $viewParams = [
                'user'              => $visitor,
                'daysBetweenChange' => $visitor->getDaysBetweenChange(true),
                'disabledButton'    => $disabledButton,
                'errorRender'       => $errorRender
            ];

            $view = $this->view('XenConcept\UsernameChange:Account\UsernameChange', 'xc_username_change_account_username_change', $viewParams);

            return $this->addAccountWrapperParams($view, 'username-change');
        }
    }

    protected function savePrivacyProcess(\XF\Entity\User $visitor)
    {
        $form = parent::savePrivacyProcess($visitor);

        $input = $this->filter('privacy', 'array');

        $userPrivacy = $visitor->getRelationOrDefault('Privacy');
        $form->setupEntityInput($userPrivacy, $input);

        return $form;
    }

    protected function preferencesSaveProcess(\XF\Entity\User $visitor)
    {
        $form  = parent::preferencesSaveProcess($visitor);

        $receiveEmails = $this->filter('option.receive_change_username_email', 'bool');

        $form->setup(function(FormAction $form) use ($receiveEmails, $visitor)
        {
            $visitor->Option->receive_change_username_email = $receiveEmails;
        });

        return $form;
    }

    /**
     * @return \XenConcept\UsernameChange\Repository\ChangeLog
     */
    protected function getChangeLogRepo()
    {
        return $this->repository('XenConcept\UsernameChange:ChangeLog');
    }

}